﻿using GeneralGommunication;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;

namespace CapSensorBase
{
    public class CapSensorCore2 : Dev7E, ICapSensorCore2
    {
        public event CoreEvent_HzMeasuredOutEventHandler HzMeasuredOut;
        public event CoreEvent_IStatusChangedEventHandler IStatusChanged;
        public event CoreEvent_DataPushedEventHandler DataPushed;

        public CapSensorCore2()
        {
            logger = NLog.LogManager.GetCurrentClassLogger();
            PrefixIndex = 1;
        }

        protected override void ParsePackAfterCheckCRC8(byte[] pack)
        {
            byte B0 = pack[0];

            // PT = 0--数据包(推送包)，1--功能包
            bool PT = Misc.MyBase.CHECKBIT(B0, 7);

            if (!PT)
            {
                //数据包
                ParseDataPack(pack);
            }
            else
            {
                ParseFuncPack(pack);
            }
        }
        protected override List<byte> GetSendPack(List<byte> datas)
        {
            datas.Insert(0, 0x80);//插入B0
            return base.GetSendPack(datas);
        }

        void ParseDataPack(byte[] buf)
        {
            if (buf.Count() < 3)
            {
                ErrCnt++;
                return;//异常
            }
            byte B0 = buf[0];

            //数据包的顺序码。 7位，肯定是1，2，3，4 排序。 如果少了一个。肯定少了一个数据包
            byte packno = B0;
            Misc.MyBase.CLEARBIT(ref packno, 7);

            //数据包类型
            byte B1 = buf[1];

            DataPushed?.Invoke(this, new CoreEvent_DataPushedEventArgs()
            {
                packno = packno,
                dataType = (char)B1
            });


            //8位 systick 单位1ms
            byte systick = buf[2];
            
            int index = 3;
            switch (B1)
            {
                case (byte)'F':
                    {
                        //CH(4b)+serial(4b)+data(2B)
                        byte channel = (byte)(buf[index] >> 4);
                        byte serial = (byte)(buf[index] & 0xF);
                        index++;
                        UInt16 data = BitConverter.ToUInt16(buf, index);
                        index += 2;

                        HzMeasuredOut?.Invoke(this, new CoreEvent_HzMeasuredOutEventArgs()
                        {
                            systick = systick,
                            channel = channel,
                            serial = serial,
                            data = data
                        });
                    }
                    break;
                case (byte)'I':
                    {
                        //0(1B)+istatus(2B)+changed(2B)
                        index++;
                        UInt16 istatus = BitConverter.ToUInt16(buf, index);
                        index += 2;
                        UInt16 ichanged = BitConverter.ToUInt16(buf, index);
                        index += 2;

                        IStatusChanged?.Invoke(this, new CoreEvent_IStatusChangedEventArgs()
                        {
                            systick = systick,
                            istatus = istatus,
                            ichanged = ichanged
                        });
                    }
                    break;
            }
        }

        [CallBack(typeof(UInt16))]
        public void GetIStatus(CallBackHandler asyncDelegate, object asyncContext)
        {
            COMMREQ commReq = new COMMREQ()
            {
                Prefix = COMMREQ.ToPrefix("IGI"),
                ResponseLen = 2,
                ParseFuncPack = (pack, dataIdx) => { return BitConverter.ToUInt16(pack, dataIdx); }
            };

            //放入 交易队列
            AddTran(
                new COMMREQ_Transaction()
                {
                    commReq = commReq,
                    asyncDelegate = asyncDelegate,
                    asyncContext = asyncContext
                });
        }
    }
}
