﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CapSensorBase
{
    /// <summary>
    /// 一个探头2个输出
    /// </summary>
    public interface ICapSensor3 : INotifyPropertyChanged
    {
        event HzMeasuredOut3EventHandler HzMeasuredOut;
        event IStatusChangedEventHandler IStatusChanged;


        /// <summary>
        /// 获取 systick 转换的时间
        /// </summary>
        DateTime Now { get; }

        /// <summary>
        /// 通道值，只有16位,更新频率为10ms
        /// </summary>
        UInt16 Data0 { get; }

        /// <summary>
        /// 通道值, 只有16位,更新频率为10ms
        /// </summary>
        UInt16 Data1 { get; }

        /// <summary>
        /// 输入口状态，只有1位, 更新频率为1ms
        /// </summary>
        UInt16 IStatus { get; }
    }

    public interface ICapSensor3Client : ICapSensor3
    {
        bool IsRunning { get; }
        /// <summary>
        /// 连接成功
        /// </summary>
        bool IsConnected { get; }

        /// <summary>
        /// 重连次数
        /// </summary>
        int ConnectCnt { get; }

        /// <summary>
        /// 已经从电容传感器获取全部当前数据
        /// </summary>
        bool IsReady { get; }

        /// <summary>
        /// 地址
        /// </summary>
        string Addr { get; set; }

        /// <summary>
        /// 连接
        /// </summary>
        void Connect();

        /// <summary>
        /// 连接 addr
        /// </summary>
        /// <param name="addr"></param>
        void Connect(string addr);

        /// <summary>
        /// 重连
        /// </summary>
        void ReConnect();

        /// <summary>
        /// 断开连接
        /// </summary>
        void Disconnect();

        /// <summary>
        ///  通过判断systick，判断AD盒是否发生重启;
        ///  BeResetTime为上一次被复位时间， 此为 电脑的时间
        /// </summary>
        DateTime BeResetTime { get; }
    }



}
