﻿using GeneralGommunication;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CapSensorBase
{
    public interface ICapSensorCore : IDev7E
    {
        event CoreEvent_HzMeasuredOutEventHandler HzMeasuredOut;
        event CoreEvent_IStatusChangedEventHandler IStatusChanged;
        event CoreEvent_DataPushedEventHandler DataPushed;

        /// <summary>
        /// frequency divider 设置分频
        /// </summary>
        void SetFreqDiv(UInt16 fd, CallBackHandler asyncDelegate, object asyncContext);

        /// <summary>
        /// 设置 是否自动计算分频数
        /// </summary>
        /// <param name="isAuto"></param>
        void SetAutoFreqDiv(bool isAuto, CallBackHandler asyncDelegate, object asyncContext);

        /// <summary>
        /// 强制读取分频数
        /// </summary>
        [CallBack(typeof(GetFreqDivResponse))]
        void GetFreqDiv(CallBackHandler asyncDelegate, object asyncContext);

        #region PID
        void SetP(float kp, CallBackHandler asyncDelegate, object asyncContext);
        void SetI(float ki, CallBackHandler asyncDelegate, object asyncContext);

        void SetD(float kd, CallBackHandler asyncDelegate, object asyncContext);

        void SetT(double target, CallBackHandler asyncDelegate, object asyncContext);

        [CallBack(typeof(float))]
        void GetP(CallBackHandler asyncDelegate, object asyncContext);

        [CallBack(typeof(float))]
        void GetI(CallBackHandler asyncDelegate, object asyncContext);

        [CallBack(typeof(float))]
        void GetD(CallBackHandler asyncDelegate, object asyncContext);

        [CallBack(typeof(double))]
        void GetT(CallBackHandler asyncDelegate, object asyncContext);
        #endregion

        /// <summary>
        /// 设置PWM 自动
        /// </summary>
        /// <param name="on"></param>
        void SetPwmAuto(bool on, CallBackHandler asyncDelegate, object asyncContext);
        /// <summary>
        /// 设置PWM 值
        /// </summary>
        /// <param name="pwm"></param>
        void SetPwm(int pwm, CallBackHandler asyncDelegate, object asyncContext);


        [CallBack(typeof(GetIStatusResponse))]
        void GetIStatus(CallBackHandler asyncDelegate, object asyncContext);
    }



    public class GetFreqDivResponse 
    {
        public UInt16 freqDiv;
        public bool isFdAuto;
    }
    public class GetIStatusResponse
    {
        public UInt16 IStatus;
    }
}
