﻿using CapSensorBase;
using FlyADBase;
using FObjBase;
using Misc;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Diagnostics.Tracing;
using System.Linq;

namespace FLY.Thick.FilmCasting.Cap.Server
{
    public class FlyAd7WithCapSensor : INotifyPropertyChanged
    {
        #region INotifyPropertyChanged 成员
        protected void NotifyPropertyChanged(string propertyName)
        {
            if (this.PropertyChanged != null)
                this.PropertyChanged.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        /// <summary>
        /// timegrid 事件 0.2s一次， 
        /// </summary>
        public event TimeGridEventHandler TimeGridEvent;

        /// <summary>
        /// minigrid 事件 grid改变，就会推送。 推送最小周期为0.1s秒
        /// </summary>
        public event MiniGridEventHandler MiniGridEvent;

        /// <summary>
        /// 
        /// </summary>
        public event TimeGridAdvEventHandler TimeGridAdvEvent;

        //一共有1.AD数据池（由timegrid提供） 1min
        //2.pos数据池（pos推送提供） 1min
        //4.当接收的grid事件数据。它有 （direction, grid_start，grid_len, systick )
        //systick 就是结束的时间点。 当AD数据池出现了这个时间点

        //pos数据池向前找。 pos 在 grid_start*posOfGrid 范围的数据。 
        //找到开始的systick 后，整合3个数据池的数据。
        //5.最后代替 grid 推送出去。
        public TimeGridAdvHelper mTimeGridAdvHelper { get; } = new TimeGridAdvHelper();

        /// <summary>
        /// 通过轮询，知道脉冲已经停下。 等AD队列也有数据，就推送 timegridadv
        /// </summary>
        public bool IsTimeToPushTimeGridAdv { get; set; }

        public IFlyADClientAdv flyad;
        public CapSensor3 capSensor;
        public DevSync devSync;

        DateTime lastUpdateADTime = DateTime.MinValue;


        /// <summary>
        /// 上一次更新grid数组的最后时刻
        /// </summary>
        DateTime lastUpdateGridEndTime = DateTime.MinValue;
        /// <summary>
        /// 上一次更新grid数组的grid序号
        /// </summary>
        int lastUpdateGrid = -1;
        /// <summary>
        /// 上一次更新grid数组的方向
        /// </summary>
        DIRECTION lastUpdateGridDirection = DIRECTION.FIX;



        /// <summary>
        /// 上一次推送grid的序号
        /// </summary>
        int? lastPushGrid = null;
        DIRECTION lastPushGridDirection = DIRECTION.FIX;

        /// <summary>
        /// 推送grid 的计时器，限制最小推送周期
        /// </summary>
        Stopwatch stopwatch_pushGridTime = new Stopwatch();


        Stopwatch stopwatch_addPosTime = new Stopwatch();

        SGrid fGrid = new SGrid();

        public void Init(IFlyADClientAdv flyad, CapSensor3 capSensor, DevSync devSync)
        {
            this.flyad = flyad;
            this.capSensor = capSensor;
            this.devSync = devSync;

            mTimeGridAdvHelper.Init();

            capSensor.HzMeasuredOut += CapSensor_HzMeasuredOut;

            devSync.PropertyChanged += DevSync_PropertyChanged;

            flyad.PositionChangedEvent += Flyad_PositionChangedEvent;
            flyad.DriveStatusChangedEvent += Flyad_DriveStatusChangedEvent;
            flyad.IStatusChangedEvent += Flyad_IStatusChangedEvent;
            fGrid.SetSize(flyad.PosLen / flyad.PosOfGrid);

            flyad.PropertyChanged += Flyad_PropertyChanged;

            //轮询事件触发
            PollModule.Current.Poll_Config(OnPoll);
        }



        private void Flyad_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if ((e.PropertyName == nameof(flyad.PosLen)) ||
                (e.PropertyName == nameof(flyad.PosOfGrid)))
            {
                fGrid.SetSize(flyad.PosLen / flyad.PosOfGrid);
            }
        }

        private void Flyad_DriveStatusChangedEvent(object sender, DriveStatusChangedEventArgs e)
        {
            if (!devSync.IsSyncSuccess)
                return;

            mTimeGridAdvHelper.AddDriveStatus(e.Time, e.DriveOrder, e.DriveStatus);
        }

        private void Flyad_PositionChangedEvent(object sender, PositionChangedEventArgs e)
        {
            if (!devSync.IsSyncSuccess)
                return;

            AddPos(e.Time, e.Position);
        }
        private void Flyad_IStatusChangedEvent(object sender, FlyADBase.IStatusChangedEventArgs e)
        {
            if (!devSync.IsSyncSuccess)
                return;
            AddPos(e.Time, e.Position);
        }
        private void DevSync_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == nameof(devSync.IsSyncSuccess))
            {
                if (devSync.IsSyncSuccess)
                {
                    //同步成功
                    afterContected();
                }
            }

        }

        void afterContected()
        {
            mTimeGridAdvHelper.Clear();

            AddPos(flyad.Now, flyad.Position);

            mTimeGridAdvHelper.AddDriveStatus(flyad.Now, flyad.DriveOrder, flyad.DriveStatus);
        }
        void AddPos(DateTime dt, int pos)
        {

            mTimeGridAdvHelper.AddPos(dt, pos);
            stopwatch_addPosTime.Restart();
        }

        /// <summary>
        /// 10ms 推送一次，放入缓存区
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CapSensor_HzMeasuredOut(object sender, HzMeasuredOut3EventArgs e)
        {
            if (!devSync.IsSyncSuccess)
                return;

            var endTime = e.time + devSync.SlaveSysTimeOffset;
            endTime = endTime - TimeSpan.FromMilliseconds(flyad.ADLag);
            //AD值是滞后的， 让它超前

            mTimeGridAdvHelper.AddAd(endTime, e.data0);
        }

        void OnPoll()
        {
            if (!devSync.IsSyncSuccess)
                return;
            OnPoll_AddPos();

            OnPoll_grid();

            OnPoll_TimeGrid();
            OnPoll_TimeGridAdv();
            OnPoll_miniGrid();
        }
        void OnPoll_AddPos()
        {
            //防止脉冲不变化， 就没有AddPos.
            //每1秒插入脉冲
            if (!stopwatch_addPosTime.IsRunning)
            {
                stopwatch_addPosTime.Restart();
                return;
            }

            if (stopwatch_addPosTime.ElapsedMilliseconds >= 1000)
            {
                AddPos(flyad.Now, flyad.Position);
            }
        }

        void OnPoll_TimeGrid()
        {
            //推送间隔0.2s
            var inteval_ts = TimeSpan.FromSeconds(0.2);

            var newestTime = mTimeGridAdvHelper.NewestTimeOfAd;
            if (flyad.CorrectADs != null)
            {
                newestTime = mTimeGridAdvHelper.NewestTimeOfValid;
            }
            
            if (newestTime == DateTime.MinValue)//还没有数据
                return;

            if (lastUpdateADTime == DateTime.MinValue)
            {
                //上次没有推送数据
                lastUpdateADTime = newestTime - inteval_ts;
            }
            else
            {
                if (newestTime - lastUpdateADTime < inteval_ts)
                    return;
            }
            DateTime beginTime = lastUpdateADTime + TimeSpan.FromMilliseconds(1);
            DateTime endTime = newestTime;
            lastUpdateADTime = newestTime;

            if (TimeGridEvent == null)
                return;


            //获取0.2s到现在的数据
            var dataList = mTimeGridAdvHelper.GetData(
                beginTime, endTime,
                out DateTime response_endTime);

            if (dataList.Count() == 0)//异常，不应该没有数据
            {
                throw new Exception($"[{beginTime}~{endTime}] 不能获取到AD数据");
            }


            //真实的开始时间
            beginTime = response_endTime - TimeSpan.FromMilliseconds(dataList.Count() - 1);

            //算AD均值
            if (dataList.Any(d => d.ad == null))
            {
                throw new Exception($"[{beginTime}~{endTime}] 有空的AD值");
            }


            if (flyad.CorrectADs != null)
            {
                for (int i = 0; i < dataList.Count(); i++)
                {
                    var d = dataList[i];
                    if (d.pos != null && d.ad != null)
                    {
                        d.ad = CorrectAD((int)d.pos / flyad.PosOfGrid, (int)d.ad);
                        dataList[i] = d;
                    }
                }
            }

            int[] adList = dataList.Select(d => (int)d.ad).ToArray();

            //推送timegrid事件
            TimeGridEvent(this,
                new TimeGridEventArgs(beginTime, TimeSpan.FromMilliseconds(1), adList));
        }

        int GetDataPoolGrid(int index)
        {
            return (int)mTimeGridAdvHelper.DataPool[index].pos / flyad.PosOfGrid;
        }

        /// <summary>
        /// 找到上一个grid的最后时刻
        /// </summary>
        /// <param name="lastValidIndex"></param>
        /// <param name="updateGridBeginIndex"></param>
        /// <param name="updateGridEndIndex"></param>
        /// <returns></returns>
        bool FindOutUpdateGridEndIndex(int lastValidIndex, int updateGridBeginIndex, out int updateGridEndIndex)
        {
            updateGridEndIndex = updateGridBeginIndex;

            //最后 有效数据 grid号
            int lastValidGrid = GetDataPoolGrid(lastValidIndex);

            int firstGrid = GetDataPoolGrid(updateGridBeginIndex);
            if (firstGrid == lastValidGrid)
            {
                //整个队列 都是同一个grid, 不处理
                return false;
            }
            else
            {
                updateGridEndIndex = updateGridBeginIndex;

                //从 有效数据 向前找到 grid号不一样的
                //肯定能找到， 最起码 [0] 是不一样的
                for (int i = lastValidIndex; i >= 0; i--)
                {
                    int grid = GetDataPoolGrid(i);
                    if (grid != lastValidGrid)
                    {
                        //找到了
                        updateGridEndIndex = i;
                        break;
                    }
                }
            }
            return true;
        }




        void OnPoll_grid()
        {
            //最后 有效数据 时间点
            DateTime lastValidTime = mTimeGridAdvHelper.NewestTimeOfValid;

            if (lastValidTime == DateTime.MinValue)//还没有 有效数据
                return;

            //最后 有效数据 数据池序号
            int lastValidIndex = mTimeGridAdvHelper.Time2Index(lastValidTime);

            //由于 Pos 或者 AD 没有更新， lastValidTime 也许在列表的前面， lastValidIndex是无效值
            if (lastValidIndex < 0)
                return;

            //当前 更新grid数组 开始的数据池序号
            int updateGridBeginIndex;

            //当前 更新grid数组 结束的数据池序号
            int updateGridEndIndex;

            //上一次更新grid数组的最后时刻
            if (lastUpdateGridEndTime == DateTime.MinValue)
            {
                //从来没有处理完数据
                updateGridBeginIndex = 0;
            }
            else
            {
                int lastUpdateGridEndIndex = mTimeGridAdvHelper.Time2Index(lastUpdateGridEndTime);
                updateGridBeginIndex = lastUpdateGridEndIndex + 1;
                if (updateGridBeginIndex < 0)
                {
                    //很久以前了，复位
                    updateGridBeginIndex = 0;
                }
            }

            //找到上一个grid的最后时刻
            bool ret = FindOutUpdateGridEndIndex(lastValidIndex, updateGridBeginIndex, out updateGridEndIndex);
            if (!ret)//没有不一样的grid序号
                return;


            int firstGrid = GetDataPoolGrid(updateGridBeginIndex);

            //最后 有效数据 grid号
            int lastValidGrid = GetDataPoolGrid(lastValidIndex);

            //方向
            Misc.DIRECTION direction;
            if (firstGrid < lastValidGrid)
                direction = Misc.DIRECTION.FORWARD;
            else
                direction = Misc.DIRECTION.BACKWARD;
            int dir_index = direction == Misc.DIRECTION.FORWARD ? 0 : 1;

            if (lastUpdateGridEndTime != DateTime.MinValue)
            {
                if (lastUpdateGridDirection != direction)
                {
                    //方向变化了，清除当前方向数据
                    fGrid.Clear(dir_index);
                }
            }

            List<int> gridList = new List<int>();
            List<int> adList = new List<int>();

            int lastGrid = -1;
            for (int i = updateGridBeginIndex; i <= updateGridEndIndex; i++)
            {
                int grid = GetDataPoolGrid(i);
                if (i != updateGridBeginIndex)
                {
                    if (grid != lastGrid)
                    {
                        if (lastGrid >= 0 && lastGrid < fGrid.data[dir_index].Length)
                        {
                            fGrid.data[dir_index][lastGrid] = (int)adList.Average();
                            gridList.Add(lastGrid);
                        }
                        adList.Clear();
                    }
                }
                var ad = mTimeGridAdvHelper.DataPool[i].ad;
                if (ad == null)
                {
                    //居然没有数据
                    throw new Exception($"DataPool[{i}].ad 没有AD数据,  grid = {grid}");
                }
                if (ad != null)
                {
                    adList.Add((int)ad);
                    //1个grid的数据不能大于2000ms
                    if (adList.Count() > 2000)
                        adList.RemoveAt(0);
                }
                lastGrid = grid;
            }

            {
                //最后一个lastGrid 肯定是满数据的。 下一个时间， grid序号就会改变， 这个是在 FindOutUpdateGridEndIndex 操作的
                if (lastGrid >= 0 && lastGrid < fGrid.data[dir_index].Length)
                {
                    if (adList.Count() > 0)
                    {
                        fGrid.data[dir_index][lastGrid] = (int)adList.Average();
                        gridList.Add(lastGrid);
                    }
                }
            }

            //这次grid填充， 检查是否连续，如果有留空， 插值填充
            for (int i = 1; i < gridList.Count(); i++)
            {
                int grid1 = gridList[i];
                int grid0 = gridList[i - 1];

                if (Math.Abs(grid1 - grid0) > 1) //有空白区
                {
                    FillEmptyGrid(dir_index, grid0, grid1);
                }
            }



            //上一次 grid 与 当前grid 中间有一段空白
            //线性插值 中间空白区
            if (lastUpdateGridEndTime != DateTime.MinValue)
            {
                if (lastUpdateGridDirection == direction)
                {
                    int grid0 = lastUpdateGrid;
                    int grid1 = firstGrid;
                    if ((grid0 >= 0 && grid0 < fGrid.data[dir_index].Length) &&
                        (grid1 >= 0 && grid1 < fGrid.data[dir_index].Length))
                    {
                        if (Math.Abs(grid0 - grid1) > 1)//有空白区
                        {
                            FillEmptyGrid(dir_index, grid0, grid1);
                        }
                    }

                }

            }


            lastUpdateGridEndTime = mTimeGridAdvHelper.Index2Time(updateGridEndIndex);
            lastUpdateGridDirection = direction;
            lastUpdateGrid = lastGrid;
        }
        /// <summary>
        /// 填充 grid0 ~grid1 之间的空白
        /// </summary>
        /// <param name="dir_index"></param>
        /// <param name="grid0"></param>
        /// <param name="grid1"></param>
        void FillEmptyGrid(int dir_index, int grid0, int grid1)
        {
            int ad0 = fGrid.data[dir_index][grid0];
            int ad1 = fGrid.data[dir_index][grid1];

            if (grid0 < grid1)
            {
                for (int i = grid0 + 1; i < grid1; i++)
                {
                    int grid = i;
                    int ad = ad0 + (ad1 - ad0) * (grid - grid0) / (grid1 - grid0);
                    fGrid.data[dir_index][grid] = ad;
                }
            }
            else
            {
                for (int i = grid1 + 1; i < grid0; i++)
                {
                    int grid = i;
                    int ad = ad0 + (ad1 - ad0) * (grid - grid0) / (grid1 - grid0);
                    fGrid.data[dir_index][grid] = ad;
                }
            }
        }
        void OnPoll_miniGrid()
        {
            if (MiniGridEvent == null)//没有注册事件
                return;

            if (lastUpdateGridEndTime == DateTime.MinValue)
                return;//没有生成过一个grid

            if (lastPushGrid == null)
            {
                //重来没有推送
            }
            else
            {
                if (lastPushGrid == lastUpdateGrid)
                {
                    return;//一样不用推送
                }

                if (stopwatch_pushGridTime.IsRunning && stopwatch_pushGridTime.Elapsed < TimeSpan.FromSeconds(0.1))
                {
                    //太快，控制推送频率
                    return;
                }
            }

            if (lastUpdateGrid >= 0 && lastUpdateGrid < fGrid.data[0].Length)
            {
                //在范围内

                int dir_index = lastUpdateGridDirection == Misc.DIRECTION.FORWARD ? 0 : 1;

                //需要推送grid
                if (lastPushGrid == null ||
                    lastPushGrid < 0 || lastPushGrid >= fGrid.data[0].Count() ||
                    lastPushGridDirection != lastUpdateGridDirection)
                {
                    //重来没有推送, 或者 之前在范围外, 或者 之前的方向与现在不一样

                    //if (lastUpdateGridDirection == DIRECTION.FORWARD)
                    //{
                    //    var buf = fGrid.data[dir_index].Take(lastUpdateGrid+1).ToArray();
                    //    MiniGridEvent(this, new MiniGridEventArgs()
                    //    {
                    //        direction = lastUpdateGridDirection,
                    //        grid_start = 0,
                    //        posOfGrid = flyad.PosOfGrid,
                    //        buf = buf
                    //    });
                    //}
                    //else 
                    //{
                    //    var buf = fGrid.data[dir_index].Skip(lastUpdateGrid).ToArray();
                    //    MiniGridEvent(this, new MiniGridEventArgs()
                    //    {
                    //        direction = lastUpdateGridDirection,
                    //        grid_start = lastUpdateGrid,
                    //        posOfGrid = flyad.PosOfGrid,
                    //        buf = buf
                    //    });
                    //}

                    var buf = new int[] { fGrid.data[dir_index][lastUpdateGrid] };
                    TriggerMiniGridEvent(new MiniGridEventArgs()
                    {
                        direction = lastUpdateGridDirection,
                        grid_start = lastUpdateGrid,
                        posOfGrid = flyad.PosOfGrid,
                        buf = buf
                    });
                }
                else
                {
                    //之前在范围内
                    if (lastUpdateGridDirection == DIRECTION.FORWARD)
                    {
                        if (lastPushGrid < lastUpdateGrid)
                        {
                            var buf = fGrid.data[dir_index].Skip((int)lastPushGrid + 1).Take(lastUpdateGrid - (int)lastPushGrid).ToArray();
                            TriggerMiniGridEvent(new MiniGridEventArgs()
                            {
                                direction = lastUpdateGridDirection,
                                grid_start = (int)lastPushGrid + 1,
                                posOfGrid = flyad.PosOfGrid,
                                buf = buf
                            });
                        }
                        else
                        {
                            //逻辑不对， 之前的推送位置 在 最新位置 右边
                            var buf = new int[] { fGrid.data[dir_index][lastUpdateGrid] };
                            TriggerMiniGridEvent(new MiniGridEventArgs()
                            {
                                direction = lastUpdateGridDirection,
                                grid_start = lastUpdateGrid,
                                posOfGrid = flyad.PosOfGrid,
                                buf = buf
                            });
                        }
                    }
                    else
                    {
                        if (lastPushGrid > lastUpdateGrid)
                        {
                            var buf = fGrid.data[dir_index].Skip(lastUpdateGrid).Take((int)lastPushGrid - lastUpdateGrid).ToArray();
                            TriggerMiniGridEvent(new MiniGridEventArgs()
                            {
                                direction = lastUpdateGridDirection,
                                grid_start = lastUpdateGrid,
                                posOfGrid = flyad.PosOfGrid,
                                buf = buf
                            });
                        }
                        else
                        {
                            //逻辑不对， 之前的推送位置 在 最新位置 左边
                            var buf = new int[] { fGrid.data[dir_index][lastUpdateGrid] };
                            TriggerMiniGridEvent(new MiniGridEventArgs()
                            {
                                direction = lastUpdateGridDirection,
                                grid_start = lastUpdateGrid,
                                posOfGrid = flyad.PosOfGrid,
                                buf = buf
                            });
                        }
                    }
                }
            }
            lastPushGridDirection = lastUpdateGridDirection;
            lastPushGrid = lastUpdateGrid;
            stopwatch_pushGridTime.Restart();
        }
        void TriggerMiniGridEvent(MiniGridEventArgs e) 
        {
            if (MiniGridEvent == null)
                return;
            flyad.CorrectADs?.Invoke(e.direction, e.grid_start,e.buf);
            MiniGridEvent.Invoke(this, e);
        }

        void OnPoll_TimeGridAdv()
        {
            if (!IsTimeToPushTimeGridAdv)
                return;

            if (TimeGridAdvEvent == null)
            {
                IsTimeToPushTimeGridAdv = false;
                return;
            }

            //触发全部高级版的 timegrid
            bool ret = mTimeGridAdvHelper.GetLastRunningTime(out DateTime beginTime, out DateTime endTime, out DRIVE_MAN_ORDER order);
            if (!ret)
                return;

            if (mTimeGridAdvHelper.NewestTimeOfValid < endTime)
            {
                //继续等，还没到
                return;
            }

            IsTimeToPushTimeGridAdv = false;


            var eventArgs = GetTimeGridAdvEvent(beginTime, endTime);

            if (eventArgs == null)
                return;//异常没有数据


            if (flyad.CorrectADs != null) 
            {
                for (int i = 0; i < eventArgs.DataList.Count(); i++) 
                {
                    var d = eventArgs.DataList[i];
                    if (d.pos != null && d.ad != null)
                    {
                        d.ad = CorrectAD((int)d.pos / flyad.PosOfGrid, (int)d.ad);
                        eventArgs.DataList[i] = d;
                    }
                }
            }
            //触发事件
            TimeGridAdvEvent?.Invoke(this, eventArgs);
        }

        /// <summary>
        /// <para>ScanMotion 使用。</para> 
        /// <para>给定按扫描的时间点。当探头停下来后，获取数据。</para> 
        /// <para>数据带滞后修正,带运动方向。</para> 
        /// </summary>
        /// <param name="beginTime"></param>
        /// <returns></returns>
        public TimeGridAdvEventArgs GetTimeGridAdvEvent(DateTime beginTime, DateTime endTime)
        {
            //获取ad列表
            var dataList = mTimeGridAdvHelper.GetData(beginTime, endTime, out DateTime response_endTime);
            if (dataList.Count() == 0)
                return null;

            endTime = response_endTime;

            Misc.DIRECTION direction;

            //判断运动方向
            if (dataList.Last().pos > dataList.First().pos)
            {
                //正向
                direction = DIRECTION.FORWARD;
            }
            else
            {
                direction = DIRECTION.BACKWARD;
            }

            ////数据滞后
            //if (!TimeGridAdvHelperExt.AdLagCorr(dataList, ref endTime, flyad.ADLag))
            //{
            //    throw new Exception("数据量太少,无法滞后修正");
            //}



            var eventArgs = new TimeGridAdvEventArgs();
            eventArgs.Direction = direction;
            eventArgs.EndTime = endTime;
            eventArgs.DataList = dataList;
            return eventArgs;
        }



        /// <summary>
        /// 从正反缓存区, 获取grid数据
        /// </summary>
        /// <param name="direction">方向, 只有 正,反</param>
        /// <param name="grid_start">grid 开始位置</param>
        /// <param name="grid_len">grid 长度</param>
        /// <param name="dat">grid 数据</param>
        public void GetGrid(Misc.DIRECTION direction, int grid_start, int grid_len, out int[] dat)
        {
            int index = 0;
            if (direction == Misc.DIRECTION.BACKWARD)
                index = 1;

            dat = new int[grid_len];

            for (int i = 0; i < dat.Length; i++)
            {
                int grid_num = grid_start + i;
                if (grid_num >= fGrid.data[index].Length)
                {
                    //剩余全部填入 NULL_VALUE;
                    for (int j = i; j < dat.Length; j++)
                    {
                        dat[i] = Misc.MyBase.NULL_VALUE;
                    }
                    break;
                }
                dat[i] = fGrid.data[index][grid_num];

            }

            flyad.CorrectADs?.Invoke(direction, grid_start, dat);
            return;
        }

        /// <summary>
        /// 从正反缓存区, 获取全部grid数据
        /// </summary>
        /// <param name="direction">方向, 只有 正,反</param>
        /// <param name="dat">grid 数据</param>
        public void GetGrid(Misc.DIRECTION direction, out int[] dat)
        {
            GetGrid(direction, 0, fGrid.data[0].Length, out dat);
        }

        int CorrectAD(int grid, int ad)
        {
            int[] d = new int[1];
            d[0] = ad;
            flyad.CorrectADs(Misc.DIRECTION.FIX, grid, d);
            int ad2 = d[0];
            return ad2;
        }
    }

    /// <summary>
    /// 正反转 grid数据 缓存区
    /// </summary>
    class SGrid
    {
        public const int GRID_MAX_SIZE = 1000;
        int size;
        public int[][] data = new int[2][];//data[0]=forword, data[1]=backward
        public SGrid()
        {
            //清空所有数据
            SetSize(GRID_MAX_SIZE);
        }
        public void SetSize(int size)
        {
            this.size = size;
            data[0] = new int[size];
            data[1] = new int[size];
            Clear();
        }
        public void Clear()
        {
            //清空所有数据
            Clear(0);
            Clear(1);
        }
        public void Clear(int index)
        {
            //清空所有数据
            for (int i = 0; i < size; i++)
            {
                data[index][i] = Misc.MyBase.NULL_VALUE;
            }
        }
    }
}
