﻿using FLY.Thick.Base.Common;
using FLY.Thick.Base.Server;
using FlyADBase;
using System;
using System.ComponentModel;
using System.Diagnostics;

namespace FLY.Thick.FilmCasting.Cap.Server
{
    public class GM_Disconnected : GM_Base
    {

        Stopwatch stopwatch = new Stopwatch();
        bool isReadyToAutoScan = false;

        GM_AutoScan gmAutoScan;
        DevSync devSync;
        DynArea dynArea;

        public GM_Disconnected()
        {
            GMState = CTRL_STATE.DISCONNECTED;

        }
        public void Init(IFlyADClientAdv flyad, DevSync devSync, DynArea dynArea, GM_AutoScan gmAutoScan)
        {
            base.Init(flyad);
            this.devSync = devSync;
            this.dynArea = dynArea;
            this.gmAutoScan = gmAutoScan;
            this.devSync.PropertyChanged += DevSync_PropertyChanged;
        }

        private void DevSync_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == nameof(devSync.IsSyncSuccess))
            {
                if (!devSync.IsSyncSuccess)
                {
                    //AD卡已经断开

                    if (dynArea.ControllerState == CTRL_STATE.SCAN ||
                        dynArea.ControllerState == CTRL_STATE.AUTOSCAN)
                    {
                        //之前在扫描中
                        isReadyToAutoScan = true;
                        stopwatch.Restart();
                    }
                    else
                    {
                        isReadyToAutoScan = false;
                        stopwatch.Stop();
                    }
                    Start();
                }
                else
                {
                    //AD盒已经重新连接上
                    if (IsRunning)//还在 CTRL_STATE.DISCONNECTED
                    {
                        if (isReadyToAutoScan)
                        {
                            //需要自动扫描
                            if (stopwatch.Elapsed < TimeSpan.FromSeconds(30))
                            {
                                isReadyToAutoScan = false;
                                stopwatch.Stop();
                                //30秒内又连上了，直接重启
                                gmAutoScan.Start(0);
                            }
                            else if (stopwatch.Elapsed < TimeSpan.FromMinutes(3))
                            {
                                isReadyToAutoScan = false;
                                stopwatch.Stop();
                                //断开连接，到现在重新连上，只过了3min，重新执行扫描
                                gmAutoScan.Start(5);
                            }
                            else
                            {
                                Stop();
                            }
                        }
                        else
                        {
                            Stop();
                        }
                    }
                }
            }
        }
        public override void Start()
        {
            IsRunning = true;
        }
        public override void Stop()
        {
            IsRunning = false;
            isReadyToAutoScan = false;
            stopwatch.Stop();
        }
    }
}
