﻿
using FlyADBase;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FLY.Thick.FilmCasting.Cap.Server
{
    /// <summary>
    /// 高级timegrid助手
    /// 使用说明：
    /// ADPool 只能储存 1min数据, 其它XXXPool 保存1.5min数据
    /// 
    /// 
    /// 获取最后一次运行动作的开始与结束时间
    /// bool ret = mTimeGridAdvHelper.GetLastRunningTime(out DateTime beginTime, out DateTime endTime, out int marker, out DRIVE_MAN_ORDER order);
    /// 
    /// 获取 指定时间范围的 timegrid 数据
    /// var adList = mTimeGridAdvHelper.GetAD(beginTime, endTime, out DateTime reponse_endTime);
    ///
    /// 获取 指定时间范围的 pos 数据
    /// var posList = mTimeGridAdvHelper.GetPos(reponse_endTime,adList.Count());
    /// 
    /// 获取 指定时间范围的 pos2 数据
    /// var pos2List = mTimeGridAdvHelper.GetPos2(reponse_endTime,adList.Count());
    /// 
    /// 获取 指定时间范围的 输入口 数据
    /// var istatusList = mTimeGridAdvHelper.GetIStatus(reponse_endTime,adList.Count());
    /// 
    /// 转为 grid 数据
    /// int posLen = 8900;
    /// var grid = ToGrid(adList, posList, posOfGrid = 10, gridLen = posLen/10);
    /// 
    /// </summary>
    public class TimeGridAdvHelper
    {
        NLog.Logger logger = NLog.LogManager.GetCurrentClassLogger();

        #region 一分钟数据缓存池, 目标每个AD数据（1ms） 都有对应的其它数据
        public const int POOL_SIZE_SECOND = 120;
        /// <summary>
        /// AD数据池时间间隔为1ms，数据是肯定是连续的。Add数据时，非连续，填充数据直到连续 
        /// </summary>
        public List<DateTimeUnit5> DataPool = new List<DateTimeUnit5>();

        /// <summary>
        /// DriveStatus 数据池, 肯定最少有一个数据
        /// </summary>
        public List<DateTimeUnit4> DriveStatusPool = new List<DateTimeUnit4>();
        #endregion

        /// <summary>
        /// 数据池 最后一个数据的时间
        /// </summary>
        public DateTime NewestTime = DateTime.MinValue;

        /// <summary>
        /// Math.Min(NewestTimeOfPos,NewestTimeOfAd)
        /// </summary>
        public DateTime NewestTimeOfValid 
        {
            get {
                if (NewestTimeOfPos < NewestTimeOfAd)
                    return NewestTimeOfPos;
                else
                    return NewestTimeOfAd;
            }
        }

        /// <summary>
        /// 上一次脉冲加入到队列的时间<para/>
        /// 就算AD盒中的脉冲不变化，也必须最少每1s添加脉冲数据到队列
        /// </summary>
        public DateTime NewestTimeOfPos = DateTime.MinValue;

        /// <summary>
        /// 上一次AD加入到队列的时间<para/>
        /// AD值是非常频繁插入到队列
        /// </summary>
        public DateTime NewestTimeOfAd = DateTime.MinValue;


        /// <summary>
        /// 
        /// </summary>
        public void Init()
        {
            Clear();
        }
        public int Time2Index(DateTime time) 
        {
            if (time == DateTime.MinValue)
                return -1;

            return TimeGridAdvHelperExt.Time2Index(DataPool.Count(), NewestTime, time);
        }
        public DateTime Index2Time(int index)
        {
            return TimeGridAdvHelperExt.Index2Time(DataPool.Count(), NewestTime, index);
        }
        public void AddAd( DateTime dt, int ad )
        {
            //有AD值的时间滞后， 所以一开始 会在队列的前面插入数据
            if (DataPool.Count() == 0)
            {
                //第1次添加
                DataPool.Add(new DateTimeUnit5() { ad = ad });
                NewestTime = dt;
                NewestTimeOfAd = dt;
                return;
            }

            //目标位置
            int target_index = Time2Index(dt);

            //需要添加的数据量
            int add_cnt = target_index - (DataPool.Count() - 1);

            //时间偏差不能大于2000ms
            if (Math.Abs(add_cnt) > 2000) 
            {
                //时间偏差大于2000ms,数据全部删除
                Clear();
                DataPool.Add(new DateTimeUnit5() { ad = ad });
                NewestTime = dt;
                NewestTimeOfAd = dt;
                return;
            }

            //当前数据需要放入队列的序号
            if (add_cnt > 0)
            {

                for (int i = 0; i < add_cnt; i++)
                {
                    DataPool.Add(new DateTimeUnit5());
                }
                NewestTime = dt;
            }
            else 
            {
                if (target_index < 0) 
                {
                    DateTimeUnit5 data = DataPool.First();
                    for (int i = 0; i < -target_index; i++) 
                    {
                        DataPool.Insert(0, data);
                    }
                    target_index = Time2Index(dt);
                }
            }
            
            int last_ad_index = Time2Index(NewestTimeOfAd);
            if (last_ad_index >= DataPool.Count()) 
            {
                //异常，NewestTimeOfAd 比 NewestTime 还要新
                throw new Exception($"NewestTimeOfAd (idx = {last_ad_index}) 比 NewestTime (cnt = {DataPool.Count()}) 还要新");
            }

            if (last_ad_index<0)//整个列表都没有AD值
            {
                //从来没有添加过AD值
                for (int i = 0; i <= target_index; i++) {
                    DateTimeUnit5 data = DataPool[i];
                    data.ad = ad;
                    DataPool[i] = data;
                }
            }
            else
            {
                //最后一个数据的序号
                int last_ad = (int)DataPool[last_ad_index].ad;

                for (int i = last_ad_index + 1; i <= target_index; i++)
                {
                    DateTimeUnit5 data = DataPool[i];

                    int curr_ad;
                    if (i == target_index)
                    {
                        //线性插值
                        curr_ad = (i - last_ad_index) * (ad - last_ad) / (target_index - last_ad_index) + last_ad;
                    }
                    else
                    {
                        curr_ad = ad;
                    }
                    data.ad = curr_ad;
                    DataPool[i] = data;
                }
            }
            NewestTimeOfAd = dt;
            LimitPool();
        }

        public void AddPos(DateTime dt, int pos)
        {

            if (DataPool.Count() == 0)
            {
                //第1次添加
                DataPool.Add(new DateTimeUnit5() { pos = pos });
                NewestTime = dt;
                NewestTimeOfPos = dt;
                return;
            }

            //目标位置
            int target_index = Time2Index(dt);

            //需要添加的数据量
            int add_cnt = target_index - (DataPool.Count() - 1);

            //时间偏差不能大于2000ms
            if (Math.Abs(add_cnt) > 2000)
            {
                //时间偏差大于2000ms,数据全部删除
                Clear();
                DataPool.Add(new DateTimeUnit5() { pos = pos });
                NewestTime = dt;
                NewestTimeOfPos = dt;
                return;
            }

            //当前数据需要放入队列的序号
            if (add_cnt>0)
            {
                for (int i = 0; i < add_cnt; i++)
                {
                    DataPool.Add(new DateTimeUnit5());
                }
                NewestTime = dt;
            }


            int last_pos_index = Time2Index(NewestTimeOfPos);
            if (last_pos_index >= DataPool.Count())
            {
                //异常，NewestTimeOfPos 比 NewestTime 还要新
                throw new Exception($"NewestTimeOfPos (idx = {last_pos_index}) 比 NewestTime (cnt = {DataPool.Count()}) 还要新");
            }

            if (last_pos_index < 0)//整个列表都没有AD值
            {
                //从来没有添加过Pos
                for (int i = 0; i <= target_index; i++)
                {
                    DateTimeUnit5 data = DataPool[i];
                    data.pos = pos;
                    DataPool[i] = data;
                }
            }
            else
            {
                //最后一个数据的序号
                int last_pos = (int)DataPool[last_pos_index].pos;

                for (int i = last_pos_index + 1; i <= target_index; i++)
                {
                    DateTimeUnit5 data = DataPool[i];

                    int curr_pos;
                    if (i == target_index)
                    {
                        //线性插值
                        curr_pos = (i - last_pos_index) * (pos - last_pos) / (target_index - last_pos_index) + last_pos;
                    }
                    else
                    {
                        curr_pos = pos;
                    }
                    data.pos = curr_pos;
                    DataPool[i] = data;
                }
            }
            NewestTimeOfPos = dt;
            LimitPool();
        }

        void LimitPool()
        {
            if (NewestTime == DateTime.MinValue)
                return;


            if (DataPool.Count() < (POOL_SIZE_SECOND + 30) * 1000)
            {
                //比限定的多了30秒 才会动作
                return;
            }

            //只保持1分钟数据
            int remove_cnt = DataPool.Count() - POOL_SIZE_SECOND * 1000;
            DataPool.RemoveRange(0, remove_cnt);


            //限制的时间
            var limitTime_other = NewestTime - TimeSpan.FromSeconds(POOL_SIZE_SECOND + 30);
            LimitSimplePool(DriveStatusPool, limitTime_other);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <typeparam name="T">IDateTimeUnit</typeparam>
        /// <param name="pool">缓存区</param>
        /// <param name="cap_min">缓存区容量单位min</param>
        void LimitSimplePool<T>(List<T> pool, DateTime limitTime) where T : IDateTimeUnit
        {

            //删除Pos 数据池
            int removeCnt = 0;
            for (int i = 0; i < pool.Count(); i++)
            {
                var cell = pool[i];
                if (cell.dt < limitTime)
                {
                    //需要删除
                    removeCnt = i + 1;
                }
                else
                {
                    break;
                }
            }

            if (removeCnt > 0)
            {
                //不能全部数据删除完,必须留一个

                if (removeCnt == pool.Count())
                {
                    removeCnt--;
                    if (removeCnt <= 0)
                        return;//不用删除了
                }

                pool.RemoveRange(0, removeCnt);
            }
        }



        /// <summary>
        /// 添加驱动状态, 返回true，则driveStatus变化了
        /// </summary>
        /// <param name="time"></param>
        /// <param name="order"></param>
        /// <param name="driveStatus"></param>
        public bool AddDriveStatus(DateTime time, DRIVE_MAN_ORDER order, DRIVE_MAN_STATUS driveStatus)
        {
            var pool = DriveStatusPool;
            if (pool.Count > 0)
            {
                if (pool.Last().dt > time)
                {
                    //异常，把之前的数据删除
                    pool.Clear();
                }
                else if (pool.Last().dt == time)
                {
                    //删除之前那个
                    pool.RemoveAt(pool.Count() - 1);
                }
            }

            pool.Add(new DateTimeUnit4() { dt = time, order = order, driveStatus = driveStatus });

            if (pool.Count() > 1)
            {
                return pool[pool.Count() - 2].driveStatus != driveStatus;
            }
            return false;
        }

        /// <summary>
        /// 
        /// </summary>
        public void Clear()
        {
            DataPool.Clear();
            DriveStatusPool.Clear();
            NewestTime = DateTime.MinValue;
            NewestTimeOfAd = DateTime.MinValue;
            NewestTimeOfPos = DateTime.MinValue;
        }


        /// <summary>
        /// 获取最后一次运行的时间段
        /// </summary>
        /// <param name="begin"></param>
        /// <param name="end"></param>
        /// <param name="order"></param>
        /// <returns></returns>
        public bool GetLastRunningTime(out DateTime begin, out DateTime end, out DRIVE_MAN_ORDER order)
        {
            begin = DateTime.MinValue;
            end = DateTime.MinValue;
            order = DRIVE_MAN_ORDER.IDLE;
            //1. 找 最后一个 driveStatus!=running的
            int idxOfStop = -1;
            for (int i = DriveStatusPool.Count() - 1; i >= 0; i--)
            {

                var cell = DriveStatusPool[i];
                if (cell.driveStatus != DRIVE_MAN_STATUS.RUNNING)
                {
                    //对！！它是。但继续向前找找看
                    idxOfStop = i;
                }
                else
                {
                    if (idxOfStop != -1)
                    {
                        //之前已经找到了，
                        break;
                    }
                }
            }
            if (idxOfStop == -1)
            {
                //一直在running，还没停下来
                return false;
            }

            //2. 最向前找running的
            int idxOfRunning = -1;
            for (int i = idxOfStop - 1; i >= 0; i--)
            {
                var cell = DriveStatusPool[i];
                if (cell.driveStatus == DRIVE_MAN_STATUS.RUNNING)
                {
                    //对！！它是。但继续向前找找看
                    idxOfRunning = i;
                }
                else
                {
                    //没了
                    break;
                }
            }

            if (idxOfRunning == -1)
            {
                //一直停着，没有运行过
                return false;
            }

            begin = DriveStatusPool[idxOfRunning].dt;
            end = DriveStatusPool[idxOfStop].dt;
            order = DriveStatusPool[idxOfStop].order;

            return true;
        }


        /// <summary>
        /// 获取 时间在 begin~end 的 Data数据
        /// </summary>
        /// <param name="begin">开始时间</param>
        /// <param name="end">结束时间</param>
        /// <returns></returns>
        public List<DateTimeUnit5> GetData(DateTime begin, DateTime end, out DateTime response_endTime)
        {
            return TimeGridAdvHelperExt.GetData(DataPool, NewestTime, begin, end, out response_endTime);
        }
        public List<DateTimeUnit5> GetData(DateTime begin)
        {
            return TimeGridAdvHelperExt.GetData(DataPool, NewestTime, begin);
        }

    }
    public interface IDateTimeUnit
    {
        DateTime dt { get; set; }
    }
    public struct DateTimeUnit : IDateTimeUnit
    {
        public int data;
        public DateTime dt { get; set; }
        public override string ToString()
        {
            return $"{dt.Ticks} | {data}";
        }
    }

    public struct DateTimeUnit4 : IDateTimeUnit
    {
        public DateTime dt { get; set; }
        public DRIVE_MAN_ORDER order;
        public DRIVE_MAN_STATUS driveStatus;
        public int marker;

        public override string ToString()
        {
            return $"{dt.Ticks} | {order} | {driveStatus} | {marker}";
        }
    }
    public struct DateTimeUnit5
    {
        public int? ad;
        public int? pos;

        public override string ToString()
        {
            return $"ad={ad} | p={pos}";
        }
    }



    public static class TimeGridAdvHelperExt
    {
        public static int Time2Index(int dataCount, DateTime newestTime, DateTime time)
        {
            int index = dataCount - 1 - (int)((newestTime - time).TotalMilliseconds);
            return index;
        }
        public static DateTime Index2Time(int dataCount, DateTime newestTime, int index) 
        {
            return newestTime - TimeSpan.FromMilliseconds(dataCount - 1 - index);
        }
        /// <summary>
        /// 导出的数据间隔是1ms 一个
        /// </summary>
        /// <param name="adPool">AD缓存区</param>
        /// <param name="begin">开始时间点</param>
        /// <returns></returns>
        public static List<DateTimeUnit5> GetData(List<DateTimeUnit5> dataPool, DateTime newestTime, DateTime begin)
        {
            int begin_idx = dataPool.Count() - (int)((newestTime - begin).TotalMilliseconds);

            if (begin_idx < 0)
                begin_idx = 0;
            if (begin_idx > dataPool.Count() - 1)
                begin_idx = dataPool.Count() - 1;

            List<DateTimeUnit5> reponse = dataPool.GetRange(begin_idx, dataPool.Count() - 1 - begin_idx + 1);

            return reponse;
        }

        /// <summary>
        /// 获取 时间在 begin~end 的 Data数据
        /// </summary>
        /// <param name="dataPool">缓存区</param>
        /// <param name="newestTime">缓存区最后一个点的时间</param>
        /// <param name="begin">开始时间点</param>
        /// <param name="end">结束时间点</param>
        /// <param name="response_endTime">导出数据最后一个点的时间</param>
        /// <returns></returns>
        public static List<DateTimeUnit5> GetData(List<DateTimeUnit5> dataPool, DateTime newestTime, DateTime begin, DateTime end, out DateTime response_endTime)
        {
            //时间转为序号
            int begin_idx = Time2Index(dataPool.Count(), newestTime, begin);

            //限制序号位置
            if (begin_idx < 0)
                begin_idx = 0;
            if (begin_idx > dataPool.Count() - 1)
                begin_idx = dataPool.Count() - 1;

            //时间转为序号
            int end_idx = Time2Index(dataPool.Count(), newestTime, end);

            //限制序号位置
            if (end_idx < 0)
                end_idx = 0;
            if (end_idx > dataPool.Count() - 1)
                end_idx = dataPool.Count() - 1;

            //开始序号 必须 小于 结束速度
            if (begin_idx > end_idx)
            {
                var t = begin_idx;
                begin_idx = end_idx;
                end_idx = t;
            }

            //实际返回数据队列 最后一个数据的时间
            response_endTime = Index2Time(dataPool.Count(), newestTime, end_idx);

            int len = end_idx - begin_idx + 1;

            List<DateTimeUnit5> reponse = dataPool.GetRange(begin_idx, len);

            return reponse;
        }

        /// <summary>
        /// 数据滞后
        /// </summary>
        /// <param name="dataPool"></param>
        /// <param name="ad1Lag"></param>
        /// <param name="ad2Lag"></param>
        public static bool AdLagCorr(List<DateTimeUnit5> dataPool, ref DateTime endTime, int ad1Lag)
        {
            int lag = ad1Lag;
            if (lag == 0)
                return true;

            int endIdx = dataPool.Count() - 1 - lag;
            if (endIdx < 0)
                return false;

            int removeCnt = lag;
            for (int i = 0; i < endIdx; i++)
            {
                int index = i;
                int idxOfAd1 = index + ad1Lag;

                if (idxOfAd1 > dataPool.Count() - 1)
                    continue;

                var data = dataPool[index];

                data.ad = dataPool[idxOfAd1].ad;

                dataPool[index] = data;
            }

            //删除没有改动的数据
            if (removeCnt > 0)
            {
                dataPool.RemoveRange(endIdx + 1, removeCnt);
                endTime -= TimeSpan.FromMilliseconds(removeCnt);
            }
            return true;
        }
        /// <summary>
        /// 获取平均速度 单位 脉冲/min <para/>
        /// 需要确保 begin 与 end 内的数据都有效，才能计算
        /// </summary>
        /// <param name="posPool">脉冲缓存区</param>
        /// <param name="begin">开始时间点</param>
        /// <param name="end">结束时间点</param>
        /// <returns></returns>
        public static double GetSpeed(List<DateTimeUnit5> dataPool, DateTime newestTime, DateTime begin, DateTime end)
        {
            
            int begin_idx = dataPool.Count() - (int)((newestTime - begin).TotalMilliseconds);
            int end_idx = dataPool.Count() - (int)((newestTime - end).TotalMilliseconds);
            if (begin_idx < 0)
                begin_idx = 0;
            if (end_idx > dataPool.Count() - 1)
                end_idx = dataPool.Count() - 1;

            var beginPoint = dataPool[begin_idx];
            var endPoint = dataPool[end_idx];
            double delta = (double)(endPoint.pos - beginPoint.pos);


            return delta / ((end_idx - begin_idx) / 60000.0);
        }

        /// <summary>
        /// 获取时间对应的 位置点 <para/>
        /// 需要确保 dt 小于 lastAddTimeOfPos 内的数据都有效，才能计算
        /// </summary>
        /// <param name="dt"></param>
        /// <param name="posPool">脉冲缓存池</param>
        /// <param name="searchIdx">查询开始序号</param>
        /// <returns></returns>
        public static int GetPos(List<DateTimeUnit5> dataPool, DateTime newestTime, DateTime dt)
        {
            int idx = dataPool.Count() - (int)((newestTime - dt).TotalMilliseconds);
            if (idx < 0)
                idx = 0;
            if (idx > dataPool.Count() - 1)
                idx = dataPool.Count() - 1;

            var point = dataPool[idx];
            return (int)(point.pos);
        }


        /// <summary>
        /// 输出 grid图。 dataPool[x+ad0Lag].ad 对应 dataPool[x].position
        /// </summary>
        /// <param name="dataPool">1ms数据缓存区</param>
        /// <param name="posOfGrid">1grid的脉冲数</param>
        /// <param name="gridLen">grid总数</param>
        /// <param name="grid_start">数据有效部分开始点</param>
        /// <param name="grid_end">数据有效部分结束点</param>
        /// <param name="gridsOfAd">导出的AD grid图</param>
        /// <param name="ad1Lag">AD 滞后</param>
        /// <param name="isMiniGrid">裁剪grid图 只导出有效部分</param>
        public static void ToGrid(
            List<DateTimeUnit5> dataPool, int posOfGrid,
            int gridLen,
            out int grid_start, out int grid_end,
            out int[] gridsOfAd,
            bool isMiniGrid = false)
        {
            int[] gridsOfAd_sum = new int[gridLen];
            int[] gridsOfAd_cnt = new int[gridLen];


            int grid_idx_last = -1;
            grid_start = Misc.MyBase.NULL_VALUE;
            grid_end = Misc.MyBase.NULL_VALUE;

            //grid_idx 应该要连续变化，不是，那就插值
            for (int j = 0; j < dataPool.Count(); j++)
            {

                if (dataPool[j].pos == null || dataPool[j].ad == null) 
                {
                    //没数据了
                    break;
                }

                int ad = (int)dataPool[j].ad;
                int pos = (int)dataPool[j].pos;

                int grid_idx = pos / posOfGrid;
                //grid不能小于0 或 大于等于 gridLen
                if (grid_idx < 0 || grid_idx >= gridLen)
                    continue;

                if (!Misc.MyBase.ISVALIDATA(grid_start))
                {
                    grid_start = grid_idx;
                    grid_end = grid_idx;
                }
                else
                {
                    if (grid_idx < grid_start)
                        grid_start = grid_idx;
                    if (grid_idx > grid_end)
                        grid_end = grid_idx;
                }

                if (grid_idx_last != -1 && (Math.Abs(grid_idx_last - grid_idx) > 1))
                {
                    //步子太大，需要插值
                    if (grid_idx_last < grid_idx)
                    {
                        for (int i = grid_idx_last; i < grid_idx; i++)
                        {
                            toGrid_step1(gridsOfAd_sum, gridsOfAd_cnt, gridLen, i, ad);
                        }
                    }
                    else
                    {
                        for (int i = grid_idx_last; i > grid_idx; i--)
                        {
                            toGrid_step1(gridsOfAd_sum, gridsOfAd_cnt, gridLen, i, ad);
                        }
                    }
                }

                toGrid_step1(gridsOfAd_sum, gridsOfAd_cnt, gridLen, grid_idx, ad);

                grid_idx_last = grid_idx;
            }

            for (int i = 0; i < gridLen; i++)
            {
                if (gridsOfAd_cnt[i] == 0)
                    gridsOfAd_sum[i] = Misc.MyBase.NULL_VALUE;
                else
                    gridsOfAd_sum[i] = (int)(gridsOfAd_sum[i] / gridsOfAd_cnt[i]);
            }

            if (isMiniGrid)
            {
                //只导出 grid_start~grid_end 数据
                if (Misc.MyBase.ISVALIDATA(grid_start))
                {
                    gridsOfAd = gridsOfAd_sum.Skip(grid_start).Take(grid_end - grid_start + 1).ToArray();
                    return;
                }
                else
                {
                    //一个数据都没有
                    gridsOfAd = null;
                    return;
                }
            }
            else
            {
                gridsOfAd = gridsOfAd_sum;
            }
        }


        static void toGrid_step1(int[] grids_sum, int[] grids_cnt, int gridLen, int grid_idx, int ad)
        {
            if ((grid_idx < 0) || (grid_idx >= gridLen))
            {
                return;
            }

            if (!Misc.MyBase.ISVALIDATA(ad))
                return;

            grids_cnt[grid_idx]++;
            grids_sum[grid_idx] += ad;
        }
    }



    /// <summary>
    /// 高级timegrid
    /// </summary>
    public class TimeGridAdvEventArgs : EventArgs
    {
        /// <summary>
        /// 
        /// </summary>
        public TimeGridAdvEventArgs()
        {

        }

        public DateTime EndTime;

        public Misc.DIRECTION Direction;

        public List<DateTimeUnit5> DataList;
    }

    /// <summary>
    /// 
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    public delegate void TimeGridAdvEventHandler(object sender, TimeGridAdvEventArgs e);

}

