﻿using System;
using System.Diagnostics;
using System.Threading.Tasks;
using System.Windows;

namespace CapSimulator
{
    /// <summary>
    /// App.xaml 的交互逻辑
    /// </summary>
    public partial class App : Application
    {
        FLY.AppHelper.AppJustOne appjustone;
        FLY.AppHelper.WindowNotifyIconHelper notifyiconhelper;

        //AD盒
        FLY.Simulation.Flyad7.FLYAD7 flyad7;

        //AD盒对外接口
        FLY.Simulation.Flyad7.OBJProxy.Flyad7_OBJProxy flyad7objproxy;

        //风环PLC
        FLY.Simulation.Blowing.KeXinPlc plcLink;

        //吹膜逻辑
        FLY.Simulation.Blowing.Blowing blowing;

        FLY.Simulation.Blowing.GageAD gageAd;

        /// <summary>
        /// 电容传感器
        /// </summary>
        FLY.Simulation.CapSensor.CapSensor2 capSensor;

        public App()
        {
            appjustone = new FLY.AppHelper.AppJustOne(this);

            AppDomain.CurrentDomain.UnhandledException += (s, e) =>
            {
                string err = e.ExceptionObject.ToString();
                var logger = NLog.LogManager.GetLogger("app");
                logger.Fatal(err);

                MessageBox.Show("程序出现异常，请把下面信息拍照发给厂家" + Environment.NewLine + err,
                    "异常，联系厂家",
                    MessageBoxButton.OK, MessageBoxImage.Error);
            };

            this.Startup += (s, e) =>
            {
                Init();
                var main = new MainWindow();

                var ver = new Misc.Ver();
                ver.SrcType = this.GetType();
                main.Title = ver.ToString();
                main.Init(gageAd, flyad7, capSensor, plcLink, blowing);

                this.MainWindow = main;

                notifyiconhelper = new FLY.AppHelper.WindowNotifyIconHelper(this.MainWindow, this.MainWindow.Title);
                
                this.MainWindow.Show();
            };
        }

        void Init() 
        {
            blowing = new FLY.Simulation.Blowing.Blowing();

            plcLink = new FLY.Simulation.Blowing.KeXinPlc();
            plcLink.Init(blowing, blowing.mAirRing2);

            gageAd = new FLY.Simulation.Blowing.GageAD();
            gageAd.Init(blowing);

            flyad7 = new FLY.Simulation.Flyad7.FLYAD7();

            capSensor = new FLY.Simulation.CapSensor.CapSensor2();

            flyad7.Init(gageAd);

            flyad7objproxy = new FLY.Simulation.Flyad7.OBJProxy.Flyad7_OBJProxy(0, flyad7);
            flyad7objproxy.Listen();


            capSensor.Init(gageAd);
            capSensor.Start();

            FObjBase.PollModule.Current.Start();


            Task.Factory.StartNew(() =>
            {
                Stopwatch stopwatch = new Stopwatch();
                stopwatch.Start();

                while (true)
                {
                    for (int i = 0; i < 50; i++)
                    {
                        // 它们内部的 systick++
                        flyad7.OnPoll();
                        capSensor.OnPoll();
                    }//触发50次

                    //计算能睡多久
                    int millisecondsDelay = (int)(flyad7.GetSysTick() - stopwatch.ElapsedMilliseconds);
                    if (millisecondsDelay > 0)
                        Task.Delay(millisecondsDelay).Wait();
                    else
                        Task.Delay(0).Wait();
                }
            });
        }
    }
}
