﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Net;
using System.Windows.Threading;

namespace FLY.Simulation.Blowing
{


    //D1 变频器当前频率
    //D200 通道数量
    //D400 设置值 160个

    //M0 心跳信号， 由电脑写1, PLC写0， 当电脑 1分钟 都没写一次1，就会停止当前全部加热 
    //M3 风环开关 没有用了
    //M4 电流互感开关
    //M5 正方向运行(顺时针旋转 clockwise)
    //M6 反方向运行(逆时针旋转 counter-clockwise)
    //M7 正转向信号(撞了它,变为 正方向运行,它是短时间信号) 
    //M8 反转向信号(撞了它,变为 反方向运行,它是短时间信号) 
    //M9 复位信号 (撞了它后， 证明在中间)

    public class KeXinPlc : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        const int ADDR_ChannelCnt = 200;
        const int ADDR_FreqRawData = 1;
        const int ADDR_ChannelValues = 400;
        const int ADDR_ChannelValues_Count = 160;

        const int ADDR_Heatbeat = 0;
        const int ADDR_PcOnline = 3;
        const int ADDR_HasCurrent = 4;

        const int ADDR_IsForw = 5;
        const int ADDR_IsBackw = 6;
        const int ADDR_IsTurnSign0 = 7;
        const int ADDR_IsTurnSign1 = 8;
        const int ADDR_IsOrgSign = 9;


        /// <summary>
        /// 
        /// </summary>
        public bool HasElectricity { get; set; }

        /// <summary>
        /// 当前电机频率
        /// </summary>
        public UInt16 CurrRotaryFreqRawData { get; protected set; }


        /// <summary>
        /// 正方向运行 （顺时针旋转 clockwise)
        /// </summary>
        public bool IsForwSign { get; protected set; }

        /// <summary>
        /// 反方向运行 (逆时针旋转 counter-clockwise)
        /// </summary>
        public bool IsBackwSign { get; protected set; }

        /// <summary>
        /// 转向信号0 撞了它， 变为 正方向运行 
        /// </summary>
        public bool IsTurnSign0 { get; protected set; }

        /// <summary>
        /// 复位原点信号 (撞了它后， 证明在中间)
        /// </summary>
        public bool IsOrgSign { get; protected set; }

        /// <summary>
        /// 转向信号1 撞了它， 变为 反方向运行 
        /// </summary>
        public bool IsTurnSign1 { get; protected set; }

        public bool[] IsChannelErrs { get; protected set; } = null;

        public bool IsAnyChannelErr { get; set; }
        public string ChannelErrMsg { get; protected set; }
        public DateTime LastUpdateDt { get; protected set; }
        public bool IsConnected { get; protected set; }

        public string PlcAddr { get; set; } = "127.168.50.60:502";
        Blowing blowing;
        Blowing.AirRing airRing;
        DispatcherTimer timer;
        Modbus.WithThread.ServerTCP mserver;


        public KeXinPlc()
        {

        }



        public void Init(Blowing blowing, Blowing.AirRing airRing)
        {
            if (!Load())
                Save();

            this.blowing = blowing;
            this.airRing = airRing;


            IPEndPoint ep = Misc.StringConverter.ToIPEndPoint(PlcAddr, IPAddress.Parse("192.168.251.60"), 502);

            mserver = new Modbus.WithThread.ServerTCP(ep, GetValue, SetValue);
            mserver.Start();


            Misc.BindingOperations.SetBinding(blowing, nameof(blowing.IsOrgSign), this, nameof(IsOrgSign));

            Misc.BindingOperations.SetBinding(blowing, nameof(blowing.IsTurnSign0), this, nameof(IsTurnSign0));
            Misc.BindingOperations.SetBinding(blowing, nameof(blowing.IsTurnSign1), this, nameof(IsTurnSign1));

            Misc.BindingOperations.SetBinding(blowing, nameof(blowing.IsForwSign), this, nameof(IsForwSign));
            Misc.BindingOperations.SetBinding(blowing, nameof(blowing.IsBackwSign), this, nameof(IsBackwSign));


            Misc.BindingOperations.SetBinding(blowing, nameof(blowing.CurrRotaryFreqRawData), updateCurrRotaryFreqRawData);

            IsAnyChannelErr = true;
            updateIsChannelErrs();


            this.PropertyChanged += PlcLink_PropertyChanged;

            timer = new DispatcherTimer();
            timer.Interval = TimeSpan.FromSeconds(0.5);
            timer.Tick += Timer_Tick;
            timer.Start();
        }
        void GetValue(int addr, object values)
        {
            if (values is UInt16[])
            {
                var uint16s = values as UInt16[];
                for (int i = 0; i < uint16s.Count(); i++)
                {
                    uint16s[i] = GetValue_reg(i + addr);
                }
            }
            else if (values is bool[])
            {
                var bools = values as bool[];
                for (int i = 0; i < bools.Count(); i++)
                {
                    bools[i] = GetValue_coil(i + addr);
                }
            }
        }

        UInt16 GetValue_reg(int addr)
        {
            if (addr == ADDR_ChannelCnt)
            {
                return (UInt16)airRing.ChannelCnt;
            }
            else if (addr == ADDR_FreqRawData)
            {
                return CurrRotaryFreqRawData;
            }
            else if (addr >= ADDR_ChannelValues && addr < ADDR_ChannelValues + ADDR_ChannelValues_Count)
            {
                int idx = addr - ADDR_ChannelValues;
                if (idx >= 0 && idx < airRing.Heats.Count())
                    return (UInt16)airRing.Heats[idx];
                else
                    return 0;
            }
            else
                return 0;

        }

        bool GetValue_coil(int addr)
        {
            switch (addr) 
            {
                case ADDR_PcOnline:
                    return true;
                case ADDR_HasCurrent:
                    return HasElectricity;
                case ADDR_IsForw:
                    return IsForwSign;
                case ADDR_IsBackw:
                    return IsBackwSign;
                case ADDR_IsTurnSign0:
                        return IsTurnSign0;
                case ADDR_IsTurnSign1:
                        return IsTurnSign1;
                case ADDR_IsOrgSign:
                        return IsOrgSign;
                default:
                    return false;
            }
        }

        void SetValue(int addr, object values)
        {

            if (values is UInt16[])
            {
                var uint16s = values as UInt16[];
                for (int i = 0; i < uint16s.Count(); i++)
                {
                    SetValue_reg(i + addr, uint16s[i]);
                }
                SetValue_reg_after();
            }
            else if (values is bool[])
            {
                var bools = values as bool[];
                for (int i = 0; i < bools.Count(); i++)
                {
                    SetValue_coil(i + addr, bools[i]);
                }
            }


        }
        /// <summary>
        /// 设置寄存器后
        /// </summary>
        void SetValue_reg_after()
        {
            airRing.SetHeatWait();
        }


        void SetValue_reg(int addr, UInt16 reg)
        {
            if (addr == ADDR_ChannelCnt)
            {
                airRing.SetChannelCnt(reg);
                if (IsChannelErrs != null && IsChannelErrs.Count() != airRing.ChannelCnt)
                {
                    updateIsChannelErrs();
                }
            }
            else if (addr >= ADDR_ChannelValues && addr < ADDR_ChannelValues + ADDR_ChannelValues_Count)
            {
                int idx = addr - ADDR_ChannelValues;
                airRing.SetHeatAsync(idx, reg);
            }
            else
            {

            }
        }
        void SetValue_coil(int addr, bool coil)
        {
            if (addr == ADDR_Heatbeat)
            {
                if (coil == true)
                {
                    LastUpdateDt = DateTime.Now;
                    IsConnected = true;
                }
            }
        }

        void updateCurrRotaryFreqRawData()
        {
            Random r = new Random();
            int range = blowing.CurrRotaryFreqRawData / 500;
            int rawdata = blowing.CurrRotaryFreqRawData + r.Next(range) - range / 2;
            CurrRotaryFreqRawData = (UInt16)(rawdata < 0 ? 0 : rawdata);


        }
        void updateIsChannelErrs()
        {
            if (IsAnyChannelErr)
            {
                SetChannelErrs();
            }
            else
            {
                ClearChannelErrs();
            }
        }
        void ClearChannelErrs()
        {
            IsChannelErrs = null;
            ChannelErrMsg = "[]";
        }

        Random random = new Random();
        void SetChannelErrs()
        {
            bool[] isChannelErrs = new bool[airRing.ChannelCnt];

            int u8s_len = (int)Math.Ceiling(isChannelErrs.Length / 8.0);

            byte[] u8s = new byte[u8s_len];
            List<int> indexs = new List<int>();
            for (int i = 0; i < 5; i++)
            {
                int index = random.Next(0, isChannelErrs.Length - 1);
                isChannelErrs[index] = true;

                int u8s_index = index / 8;
                int bit_index = index % 8;
                Misc.MyBase.SIGNBIT(ref u8s[u8s_index], bit_index);
                indexs.Add(index);
            }
            indexs.Sort();
            ChannelErrMsg = Newtonsoft.Json.JsonConvert.SerializeObject(indexs);
            IsChannelErrs = isChannelErrs;
        }
        private void Timer_Tick(object sender, EventArgs e)
        {
            OnPoll();
        }

        private void PlcLink_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == nameof(IsConnected))
            {
                if (IsConnected == true)
                    return;

                //清除全部加热
                airRing.Clear();
                HasElectricity = false;
            }
            else if (e.PropertyName == nameof(IsAnyChannelErr))
            {
                updateIsChannelErrs();
            }
        }

        //0.1s 检查一次, 
        void OnPoll()
        {
            HasElectricity = CheckHasElectricity();
            if (DateTime.Now - LastUpdateDt > TimeSpan.FromMinutes(1))
            {
                IsConnected = false;
            }

            updateCurrRotaryFreqRawData();
        }

        bool CheckHasElectricity()
        {
            bool hasElectricity = false;
            for (int i = 0; i < airRing.Heats.Count(); i++)
            {

                if (IsChannelErrs != null && i < IsChannelErrs.Count() && IsChannelErrs[i])
                    continue;

                if (airRing.Heats[i] > 0)
                {
                    hasElectricity = true;
                    break;
                }
            }
            return hasElectricity;
        }

        string filePath = "plc.json";
        public bool Save()
        {
            return KeXinPlcJsonDb.Save(this, filePath);
        }
        bool Load()
        {
            return KeXinPlcJsonDb.Load(this, filePath);
        }
    }

    public class KeXinPlcJsonDb
    {
        public static bool Load(KeXinPlc src, string filePath)
        {
            if (!File.Exists(filePath))
                return false;

            try
            {
                string json = File.ReadAllText(filePath);
                var jsonDb = JsonConvert.DeserializeObject<KeXinPlcJsonDb>(json);
                src.PlcAddr = jsonDb.PlcAddr;
                return true;
            }
            catch
            {
                //异常，没有json 解码失败
                return false;
            }

        }

        public static bool Save(KeXinPlc src, string filePath)
        {
            try
            {
                var jsonDb = new KeXinPlcJsonDb()
                {
                    PlcAddr = src.PlcAddr
                };
                string json = JsonConvert.SerializeObject(jsonDb, Formatting.Indented);
                File.WriteAllText(filePath, json);
                return true;
            }
            catch
            {
                //异常，没有json 编码失败
                return false;
            }
        }
        public string PlcAddr { get; set; } = "0.0.0.0:502";
    }
}
