﻿using GeneralGommunication;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Net.Sockets;
using System.Text;
using System.Threading.Tasks;

namespace FLY.Simulation.CapSensor
{
    public class CapSensor2:INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        public string Addr { get; set; } = "127.168.251.12:20020";
        public DateTime Now { get; private set; }

        public bool SyncIn { get; private set; } = false;
        public int Hz { get; private set; }
        public UInt32 Systick { get; private set; }

        public CapSensorCore2 Core => core;

        CapSensorCore2 core;

        GComm_TcpListen listen;
        List<GComm_TcpServer> comms = new List<GComm_TcpServer>();
        ISimulationGageAD mSimGageAD;
        bool lastSyncIn = false;
        byte pushHzCnt = 0;
        


        public CapSensor2()
        {
            core = new CapSensorCore2();

            listen = new GComm_TcpListen();

        }
        public void Init(ISimulationGageAD mSimGageAD) 
        {
            this.mSimGageAD = mSimGageAD;

            core.Csm.StartMeasure();

            //注册发送动作，把数据通过 comm模块发送出去
            core.SendMsgEvent += Core_SendMsgEvent;

            listen.NewConnection += Listen_NewConnection;
        }

        public void Start()
        {
            listen.Addr = Addr;

            listen.Start();
        }

        private void Core_SendMsgEvent(object sender)
        {
            byte[] msg = core.GetSendMsg();
            if (msg == null)
                return;

            //TODO
            //不管是谁连上，都会收到, 是BUG, 但只是模拟器而已，不管了
            foreach (var comm in comms)
            {
                comm?.Write(msg);
            }
        }

        private void Listen_NewConnection(Socket socket)
        {
            GComm_TcpServer comm = new GComm_TcpServer(socket);
            comm.PropertyChanged += Comm_PropertyChanged;

            comm.DataReceived += Comm_DataReceived;
            comms.Add(comm);
            comm.Start();

        }

        private void Comm_DataReceived(IGeneralComm sender, byte[] buf)
        {
            core.RecMsg(buf);
        }

        private void Comm_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            var comm = sender as GComm_TcpServer;

            if (e.PropertyName == nameof(GComm_TcpServer.IsRunning))
            {
                if (!comm.IsRunning)
                {
                    comm.PropertyChanged -= Comm_PropertyChanged;

                    comm.DataReceived -= Comm_DataReceived;


                    //已经断开，删除
                    comms.Remove(comm);
                }
            }
        }

        Random random = new Random();
        /// <summary>
        /// 1ms 触发一次
        /// </summary>
        public void OnPoll()
        {
            if (Now == DateTime.MinValue)
            {
                Now = DateTime.Now;
                Systick = 125;
            }
            else
            {
                Now += TimeSpan.FromTicks(TimeSpan.TicksPerMillisecond);
                Systick += 1;
            }

            int hz = mSimGageAD.GetHz();
            int p = random.Next(200);
            hz = hz - 100 + p;
            Hz = hz;
            SyncIn = mSimGageAD.GetSyncIn();

            pushData();
        }

        int hz_sum = 0;
        int ms_cnt = 0;
        void pushData() 
        {
            byte systick = (byte)Systick;

            hz_sum += Hz;
            ms_cnt++;
            if (ms_cnt >= 10) 
            {
                UInt16 data = (UInt16)(hz_sum / 1000);
                hz_sum = hz_sum % 1000;
                ms_cnt = 0;
                core.PushHz(systick, 0, pushHzCnt, data);
                pushHzCnt++;
            }

            if (lastSyncIn != SyncIn)
            {
                UInt16 istatus, ichanged;
                
                if (SyncIn) {
                    istatus = 0x0;
                    ichanged = 0x1;
                }
                else {
                    istatus = 0x1;
                    ichanged = 0x1;
                }
                core.PushIn(systick, istatus, ichanged);
                lastSyncIn = SyncIn;
            }
        }
    }
}
